10 ' Program written by : SEAN O'CONNOR
20 '------------------------------------------------------------------------
30 '|            Terminal plotting program (October 1982)                  |
40 '------------------------------------------------------------------------
50 ' Plot  the points  { ( xi, yi )  |  i = 1 . . . n } on a  graph.  The
60 ' points are read  from a  disk  file and are  stored in  the sequence
70 ' x1, y1, x2, y2, . . ., xn, yn.  They are plotted on the CRT  or line
80 ' printer.  The x axis points down the length of the page.  The y axis
90 ' points to the right and is scaled so that it extends from min { yi } 
100 ' to max { yi }.  To print on the line printer, change "print" to "lprint"
110 '
120 ' The layout of the plot is as follows:
130 '
140 '     X                            Y
150 '            -2.33E22                        3.33E4    (ymin and ymax)
160 '  -1.11E22 .--------------------------------------.
170 '  -1.12E22 |      +                               | 
180 '            |<-->| Skip%
190 '  |<------------ PageWidth ---------------------->|
200 '  |<---->|  |<----------- PlotWidth ------------>|
210 ' number width
220 '
230    GOSUB 660   'ChangeParameters            {set plot shape and size}
240    GOSUB 1010   'ReadPoints                  {read points}
250    GOSUB 1240   'Scale
260    GOSUB 1380   'CalculatePlotDimensions
270    GOSUB 1500   'PrintPlotHeading            {begin plotting}
280    GOSUB 1680  'PrintYMaxAndMin
290    GOSUB 1870  'PrintStartOfPlotFrame
300    GOSUB 2030  'PrintPlot
310    GOSUB 2330  'PrintEndOfPlotFrame
320    STOP
330 '------------------------------------------------------------------------
340 '|       Procedure PrintRowOfChar( N% ; Char$ )                         |
350 '------------------------------------------------------------------------
360 ' Print a row of characters Char$ to the terminal, N% in length. 
370 '
380   TEMPI% = I%
390 '
400   FOR I% = 1 TO N%
410       PRINT CHAR$;
420   NEXT I%
430 '
440   I% = TEMPI%
450   RETURN
460 '------------------------------------------------------------------------
470 '|         Procedure PrintNumber( x; NumSigFig% )                       |
480 '------------------------------------------------------------------------
490 ' Print the number x in scientific notation with NumSigFig significant
500 ' digits.
510 '
520 '
530   TEMPI% = I%
540 '
550   FORMAT$ = ""
560   FOR I% = 1 TO NUMSIGFIG% - 1 
570       FORMAT$ = FORMAT$ + "#" 
580   NEXT I%
590 '
600   FORMAT$ = "#." + FORMAT$ + "^^^^"
610 '
620   PRINT USING FORMAT$; X;   
630 '
640   I% = TEMPI%
650   RETURN
660 '------------------------------------------------------------------------
670 '| Procedure ChangeParameters( PlotSymbol$, PageWidth%, NumSigFig% )    | 
680 '------------------------------------------------------------------------
690 ' Change the max allowed width of the plot, the symbol used for plotting
700 ' and the number of significant figures displayed.  Use default values
710 ' otherwise.
720 '
730   PLOTSYMBOL$ = "+"
740   PAGEWIDTH% = 78
750   NUMSIGFIG% = 4
760 '
770   INPUT "Change plot format? ( y = yes ) > ",COMMAND$ 
780 '
790   IF COMMAND$ <> "y" AND COMMAND$ <> "Y" THEN RETURN
800 '
810 ' repeat  
820       PRINT "Current values:" : PRINT
830       PRINT "Plot symbol =  "; PLOTSYMBOL$
840       PRINT "Page width =  "; PAGEWIDTH%
850       PRINT "Number of significant figures =  "; NUMSIGFIG% : PRINT
860 '
870       PRINT "ps = plot symbol ( a single character )"
880       PRINT "pw = page width "
890       PRINT "sf = number of significant digits to display"
900       PRINT "q = quit entering options" : PRINT
910       INPUT "Your option" ; CM$
920 '
930       IF CM$ = "ps" OR CM$ = "PS" THEN   INPUT "Plot symbol "; PLOTSYMBOL$
940       IF CM$ = "pw" OR CM$ = "PW" THEN   INPUT "Paper width "; PAGEWIDTH%
950       IF CM$ = "sf" OR CM$ = "SF" THEN   INPUT "Num sig fig "; NUMSIGFIG%
960 '
970       PRINT : PRINT
980   IF CM$ <> "q" AND CM$ <> "Q" THEN GOTO 810
990 '
1000   RETURN
1010 '------------------------------------------------------------------------
1020 '|      Procedure ReadPoints( FileName$, x, y, NumPoints% )             |
1030 '------------------------------------------------------------------------
1040 ' Ask for the name of the file containing the points and write them to
1050 ' arrays x and y.  Return the number of points NumPoints%
1060 '
1070   DIM X( 100 ) : DIM Y( 100 )
1080 '
1090   INPUT "Name of file on disk B in CAPITAL letters "; FILENAME$
1100 '
1110   PRINT "Please wait.  Reading the file."
1120 '
1130   FILENAME$ = "B:" + FILENAME$
1140   OPEN "I",1,FILENAME$
1150 '
1160   NUMPOINTS% = 0
1170   WHILE NOT EOF( 1 )
1180       NUMPOINTS% = NUMPOINTS% + 1 
1190       INPUT #1, X( NUMPOINTS% ), Y( NUMPOINTS% )
1200   WEND
1210 '
1220   CLOSE #1
1230   RETURN
1240 '------------------------------------------------------------------------
1250 '|        Procedure Scale( x, y, NumPoints ,ymin, ymax )                |
1260 '------------------------------------------------------------------------
1270 ' Find the maximum and minimum y values, ymax and ymin
1280 '
1290   YMIN = 10!^ 37   
1300   YMAX = - 10!^ 37  
1310 '
1320   FOR I% = 1 TO NUMPOINTS%
1330       TEMP = Y( I% )
1340       IF TEMP > YMAX THEN  YMAX = TEMP
1350       IF TEMP < YMIN THEN  YMIN = TEMP
1360  NEXT I%
1370  RETURN
1380 '------------------------------------------------------------------------
1390 '|           Procedure CalculatePlotDimensions                          |
1400 '------------------------------------------------------------------------
1410 ' Calculate the width of a number, the height of the y-axis and its scale.
1420 ' The number added to NumSigFig% below is explained by 5 = 1(for + or -
1430 ' sign) + 1(for decimal point) + 1(for exponent symbol E) + 
1440 ' 2(for exponent)
1450 '  
1460  NUMBERWIDTH% = NUMSIGFIG% + 5  
1470  PLOTWIDTH% = PAGEWIDTH% - ( NUMBERWIDTH% + 2 )
1480  SCALEFACTOR = PLOTWIDTH% / ( YMAX - YMIN )
1490  RETURN
1500 '------------------------------------------------------------------------
1510 '|          Procedure PrintPlotHeading                                  |
1520 '------------------------------------------------------------------------
1530 ' Display the letters  X and Y at the top, properly centered. 
1540 '    
1550   PRINT : PRINT
1560 '
1570   N% = NUMBERWIDTH% \ 2  :   CHAR$ = " "  
1580   GOSUB 330  'PrintRowOfChar
1590 '
1600   PRINT "X";
1610 '
1620   N% = (NUMBERWIDTH% \ 2) + 1 + (PLOTWIDTH% \ 2) : CHAR$ = " " 
1630   GOSUB 330 'PrintRowOfChar
1640 '
1650   PRINT "Y"; : PRINT
1660 '
1670   RETURN
1680 '-----------------------------------------------------------------------
1690 '|         Procedure PrintYMaxAndMin                                   |
1700 '-----------------------------------------------------------------------
1710 ' Display the maximum and minimum y values.
1720 '
1730   N% = NUMBERWIDTH% + 2 : CHAR$ = " "
1740   GOSUB 330 'PrintRowOfChar
1750 '
1760   X = YMIN 
1770   GOSUB 460  'PrintNumber
1780 '
1790   N% = PAGEWIDTH% - ( 3 * NUMBERWIDTH% + 2 ) : CHAR$ = " "
1800   GOSUB 330 'PrintRowOfChar
1810 '
1820   X = YMAX
1830   GOSUB 460  'PrintNumber
1840   PRINT
1850 '
1860   RETURN
1870 '-----------------------------------------------------------------------
1880 '|         Procedure PrintStartOfPlotFrame                             |
1890 '-----------------------------------------------------------------------
1900 ' Print a line of hyphens to begin the frame of the plot.
1910 '
1920   N% = NUMBERWIDTH% + 1 : CHAR$ = " "
1930   GOSUB 330 'PrintRowOfChar
1940 '
1950   PRINT ".";
1960 '
1970   N% = PLOTWIDTH% : CHAR$ = "-"
1980   GOSUB 330  'PrintRowOfChar
1990 '
2000   PRINT "."
2010 ' 
2020   RETURN
2030 '------------------------------------------------------------------------
2040 '|              Procedure PrintPlot                                     |
2050 '------------------------------------------------------------------------
2060 ' Plot the values in the graph using the plot symbol and construct a  
2070 ' frame.
2080 '
2090   FOR I% = 1 TO NUMPOINTS%
2100 '                                            ---- Plot x axis value
2110       X = X( I% )                       
2120       GOSUB  460 'PrintNumber
2130 '                                            ---- Print frame
2140       PRINT " |";
2150 '                      ---- compute num spaces to skip before plotting y
2160 '
2170       SKIP% = INT(  .5 + (Y( I% ) - YMIN) * SCALEFACTOR  )   - 1
2180       IF SKIP% = -1 THEN SKIP% = 0
2190 '
2200       N% = SKIP% : CHAR$ = " " 
2210       GOSUB 330  'PrintRowOfChar
2220 '                                            ---- plot symbol
2230       PRINT PLOTSYMBOL$;
2240 '                                            ---- skip spaces
2250 '
2260       N% = PLOTWIDTH% - (SKIP% + 1) : CHAR$ = " "
2270       GOSUB 330   'PrintRowOfChar
2280 '                                            ---- complete the frame
2290       PRINT "|"
2300 '
2310   NEXT I%
2320   RETURN
2330 '-----------------------------------------------------------------------
2340 '|          Procedure PrintEndOfPlotFrame                              |
2350 '-----------------------------------------------------------------------
2360 ' Print a line of hyphens to close off the bottom of the plot.
2370 '
2380   N% = NUMBERWIDTH% + 1 : CHAR$ = " "
2390   GOSUB 330 'PrintRowOfChar
2400   PRINT ".";
2410 '
2420   N% = PLOTWIDTH% : CHAR$ = "-"
2430   GOSUB 330  'PrintRowOfChar
2440   PRINT "."
2450   RETURN
