;******************************************************
;
;	PROGRAM ID:	SD/DISK
;
;******************************************************
;
;	VERSION:	JADE RAM DISK - RELEASE 1
;
;******************************************************
;
;	PRESENTED BY:	JADE COMPUTER PRODUCTS
;			4901 W. ROSECRANS BLVD.
;			HAWTHORNE, CALIFORNIA
;			90250,	U.S.A.
;
;******************************************************

;******************************************************
; SELECT THE EXPANDORAM CONFIGURATION IN YOUR SYSTEM
;******************************************************

TRUE	EQU	1
FALSE	EQU	0

XRAM$3	EQU	FALSE	;USING ONE EXPANDORAM-3.
XRAM$32	EQU	FALSE	;USING ONE EACH XRAM 2 & 3.
XRAM$33	EQU	TRUE	;USING TWO EXPANDORAM 3.

PORT$AD	EQU	0FFH		;EXPANDORAM 2&3 PORT.

;*****( PARAMETERS )***********************************

XD$ADDR	EQU	0F800H		;EXTENDED DRIVER ADDR.
SEC$SZ	EQU	00080H		;128 BYTES/SECTOR.
RAM$DISK EQU	2		;SELECT DRIVE C.
CNTL$Z	EQU	01AH		;CP/M END-OF-FILE.

	ORG	0100H		;T.P.A.

;******************************************************
; BLOCK MOVE RDIOS ABOVE CCP/BDOS AREA
;******************************************************

	LXI	B,BLK$SZ	;SET BLOCK SIZE.
	LXI	D,XD$ADDR	;DESTINATION.
	LXI	H,BLK$BG	;BLOCK BEGINNING.
	DB	0EDH,0B0H	;Z80 LDIR

	JMP	INIT$BG		;JUMP TO OFFSET BLOCK.

;******************************************************
; BEGGINING OF EXTENDED DRIVER - RAM DISK AND LIST
;******************************************************

OFFSET	EQU	XD$ADDR-$	;CALC ADDRESS OFFSET.
BLK$BG	EQU	$		;TPA IMAGE ADDRESS.

;******************************************************
; RAM DISK - DISK PARAMETER BLOCK - MUST FOLLOW JMPs  *
;******************************************************

RM$PBK	EQU	$+OFFSET

	IF	XRAM$3
	DW	192	;SECTORS PER TRACK.
	DB	3	;BLOCK SHIFT FACTOR.
	DB	7	;BLOCK MASK.
	DB	0	;NULL MASK.
	DW	4*48-1	;DISK SIZE - 1.
	DW	63	;DIRECTORY MAX.
	DB	11000000B ;ALLOC 0.
	DB	0	;ALLOC 1.
	DW	0	;CHECK SIZE 0 FOR RAM.
	DW	2	;TRACK OFFSET (BANK 0).
	ENDIF

	IF	XRAM$32
	DW	192	;SECTORS PER TRACK.
	DB	3	;BLOCK SHIFT FACTOR.
	DB	7	;BLOCK MASK.
	DB	0	;NULL MASK.
	DW	5*48-1	;DISK SIZE - 1.
	DW	63	;DIRECTORY MAX.
	DB	11000000B ;ALLOC 0.
	DB	0	;ALLOC 1.
	DW	0	;CHECK SIZE 0 FOR RAM.
	DW	2	;TRACK OFFSET (BANK 0).
	ENDIF

	IF	XRAM$33
	DW	192	;SECTORS PER TRACK.
	DB	4	;BLOCK SHIFT FACTOR.
	DB	15	;BLOCK MASK.
	DB	1	;NULL MASK.
	DW	9*24-1	;DISK SIZE - 1.
	DW	63	;DIRECTORY MAX.
	DB	10000000B ;ALLOC 0.
	DB	0	;ALLOC 1.
	DW	0	;CHECK SIZE 0 FOR RAM.
	DW	2	;TRACK OFFSET (BANK 0).
	ENDIF

;******************************************************
; INITIALIZATION AREA OF RAMDISK BIOS AREA.
;******************************************************

INIT$BG	EQU	$+OFFSET	;DECLARE ADDRESS.

;******************************************************
; EXTRACT IMAGE OF BIOS JUMP TABLE
;******************************************************

	LXI	B,BIOS$JS	;SET BYTE COUNT.
	LXI	D,BIOS$JT	;DESTINATION NEW JT.
	LHLD	1		;LOAD ADDR OF WARM BOOT
	DCX H!	DCX H!	DCX H	;COLD BOOT ADDR NOW.
	DB	0EDH,0B0H	;Z80 BLOCK MOVE.

;******************************************************
; ALTER BIOS JUMP TABLE TO LINK RAM/DISK IOS
;******************************************************

	LHLD	1		;LOAD ADDR OF WARM BOOT
	LXI	D,B$LIST-BIOS$JT-3	;VECTOR OFFSET
	DAD	D		;NOW POINT TO LIST VEC.
	XCHG			;DESTINATION TO DE REG.
	LXI	H,XIOS$JT	;IMAGE TO MOVE TO BIOS.
	LXI	B,XIOS$JS	;SIZE OF PARTIAL TBL.
	DB	0EDH,0B0H	;Z80 BLOCK MOVE.
	JMP	0		;EXIT XIOS INIT.

;******************************************************
; XIOS RAM DISK LINKAGES
;******************************************************

X$HOME	EQU	$+OFFSET
	XRA	A		;ZERO A REGISTER.
	STA	T$TRAK		;STORE TRACK VALUE.
	LDA	T$DRIV		;GET DRIVE.
	CPI	RAM$DISK	;TEST IF RAM DISK.
	RZ			;IF SO, EXIT NOW.
	JMP	B$HOME		;GOTO PROM, BYE.

X$PICK	EQU	$+OFFSET
	MOV	A,C		;DRIVE NMBR TO ACUM.
	STA	T$DRIV		;STORE DRIVE NMBR.
	CPI	RAM$DISK	;TEST IF RAM DISK.
	JNZ	B$PICK		;EXIT IF FLOPPY DRV.
	LXI	H,RM$DPH	;ADDR DRV PARA HDR.
	RET			;RETURN TO BDOS.

X$TRAK	EQU	$+OFFSET
	MOV	A,C		;TRACK NMBR TO A.
	STA	T$TRAK		;STORE TRACK NMBR.
	JMP	B$TRAK		;GOTO BIOS PROM.

X$SECT	EQU	$+OFFSET
	MOV	A,C		;SECTOR NMBR TO A.
	STA	T$SECT		;STORE SECTOR NMBR.
	JMP	B$SECT		;GOTO BIOS PROM.
	
X$ADDR	EQU	$+OFFSET
	MOV	H,B		;HIGH ORDER BC >> HL.
	MOV	L,C		;LOW  ORDER BC >> HL.
	SHLD	T$ADDR		;STORE TRANSFER ADDR.
	JMP	B$ADDR		;GOTO BIOS PROM.

X$TRNS	EQU	$+OFFSET
	LDA	T$DRIV		;LOAD DRIVE NMBR.
	CPI	RAM$DISK	;SEE IF RAM DISK.
	JNZ	B$TRNS		;IF NOT THEN BIOS PROM.
	MOV	H,B		;HIGH ORDER SECTOR
	MOV	L,C		;LOW  ORDER SECTOR.
	RET			;RETURN UNTRANSLATED.

;******************************************************
; READ SECTOR FROM RAM DISK
;******************************************************

X$READ	EQU	$+OFFSET
	LDA	T$DRIV		;GET DRIVE NUMBER.
	CPI	RAM$DISK	;SEE IF RAM/DRIVE.
	JNZ	B$READ		;IF FLOPPY, GOTO PROM.

	LXI	B,192		;SECTORS/TRACK.
	LXI	H,SEC$BF	;LOAD SECTOR BUFF ADDR.
	XCHG			;HL >> DE.
	LXI	H,0		;ZERO HL.

	LDA	T$SECT		;SELECTED SECTOR.
	MOV	L,A		;HL IS SECTOR NMBR.
	LDA	T$TRAK		;LOAD TRACK NMBR.
	RRC			;ROTATE RIGHT.
	JNC	X$REA2		;SKIP IF EVEN TRACK.
	DAD	B		;UPPER 24K ODD TRACK.
X$REA2	EQU	$+OFFSET
	DAD H!	DAD H		;*2  AND *4.
	DAD H!	DAD H		;*16 AND *32.
	DAD H!	DAD H		;*32 AND *64.
	DAD H			;*128 (ADDR OF SECTOR)
	ANI	0FH		;KEEP BANK # REASONABLE
	LXI	B,SEC$SZ	;SET SECTOR SIZE.
	OUT	PORT$AD		;SELECT BANK TO READ.
	DB	0EDH,0B0H	;BLOCK MOVE COMMAND.
	MVI	A,0		;GOOD OLD BANK 0.
	OUT	PORT$AD		;SELECT IT.

	LXI	B,SEC$SZ	;128 BYTES.
	LHLD	T$ADDR		;SECTOR ADDRESS. 
	XCHG			; HL >> DE.
	LXI	H,SEC$BF	;LOAD SECTOR BUFF ADDR
	DB	0EDH,0B0H	;BLOCK TRANSFER Z80.
	XRA	A		;ZERO A REGISTER.
	RET			;NORMAL EXIT.

;******************************************************
; WRITE SECTOR TO RAM DISK
;******************************************************

X$WRIT	EQU	$+OFFSET
	LDA	T$DRIV		;GET DRIVE NUMBER.
	CPI	RAM$DISK	;SEE IF RAM/DRIVE.
	JNZ	B$WRIT		;IF FLOPPY, GOTO PROM.

	LXI	B,SEC$SZ	;128 BYTES.
	LXI	H,SEC$BF	;LOAD SECTOR BUFF ADDR.
	XCHG			; HL >> DE.
	LHLD	T$ADDR		;SECTOR ADDRESS. 
	DB	0EDH,0B0H	;BLOCK TRANSFER Z80.

	LXI	B,192		;SECTORS/TRACK.
	LXI	H,0		;ZERO HL.
	LDA	T$SECT		;SELECTED SECTOR.
	MOV	L,A		;HL IS SECTOR NMBR.
	LDA	T$TRAK		;LOAD TRACK NMBR.
	RRC			;ROTATE RIGHT.
	JNC	X$WRI2		;SKIP IF EVEN TRACK.
	DAD	B		;UPPER 24K ODD TRACK.
X$WRI2	EQU	$+OFFSET
	DAD H!	DAD H		;*2  AND *4.
	DAD H!	DAD H		;*16 AND *32.
	DAD H!	DAD H		;*32 AND *64.
	DAD H			;*128 (ADDR OF SECTOR)
	XCHG			;DE IS SECTOR ADDRESS.
	ANI	0FH		;KEEP BANK # REASONABLE
	LXI	B,SEC$SZ	;SET SECTOR SIZE.
	LXI	H,SEC$BF	;SECTOR BUFFER ADDR.

	OUT	PORT$AD		;SELECT BANK TO READ.
	DB	0EDH,0B0H	;BLOCK MOVE GET SECTOR.
	MVI	A,0		;GOOD OLD BANK 0.
	OUT	PORT$AD		;SELECT IT.

	XRA	A		;ZERO A REGISTER.
	RET			;NORMAL EXIT.

;******************************************************
; PRINTER DRIVER AREA - SBC-200 PARALLEL PORT DRIVER  *
;******************************************************

X$LIST	EQU	$+OFFSET	;LIST DRIVER ENTRY.
X$PNCH	EQU	$+OFFSET	;PUNCH DRIVER ENTRY.
	IN	07FH		;GET STATUS
	ANI	001H		;TEST FLAG
	XRI	001H		;INVERT BIT
	JNZ	X$LIST		;REPEAT NOT READY.
	MOV	A,C		;LOAD CHARACTER
	OUT	07EH		;OUTPUT CHAR.
	MVI	A,00		;SET CHAR STROBE
	OUT	07FH		;OUTPUT IT.
	MVI	A,01		;RESET CHAR STROBE.
	OUT	07FH		;OUTPUT RESET
	RET			;RETURN FROM CALL.

X$LSTS	EQU	$+OFFSET	;LIST STATUS ENTRY.
	IN	07FH		;GET STATUS
	ANI	001H		;TEST FLAG
	XRI	001H		;INVERT BIT
	JNZ	X$LST2		;PRINTER BUSY JMP
	DCR	A		;ACUM TO FF HEX
	RET			;RETURN FROM CALL.
X$LST2	EQU	$+OFFSET
	XRA	A		;ZERO ACUM
	RET			;RETURN FROM CALL

X$PRDR	EQU	$+OFFSET	;DUMMY READER CALL.
	MVI	A,CNTL$Z	;END OF FILE CHAR.
	RET			;RETURN TO CALLER.

;******************************************************
; NEW JUMP VECTORS FOR BIOS - TO LINK XIOS
;******************************************************

XIOS$JT	EQU	$+OFFSET
	JMP	X$LIST		;LISTING OUTPUT.   
	JMP	X$PNCH		;PUNCH OUTPUT.     
	JMP	X$PRDR		;READER INPUT.     
	JMP	X$HOME		;RESTORE DRIVE.    
	JMP	X$PICK		;SELECT DRIVE.     
	JMP	X$TRAK		;SELECT TRACK.     
	JMP	X$SECT		;SELECT SECTOR.    
	JMP	X$ADDR		;SET ADDRESS.
	JMP	X$READ		;READ A SECTOR.    
	JMP	X$WRIT		;WRITE A SECTOR.   
	JMP	X$LSTS		;CHECK LIST STAT.
	JMP	X$TRNS		;TRANSLATE SECT. 
XIOS$JS	EQU	$+OFFSET - XIOS$JT

;******************************************************
; BIOS IMAGE - JUMP VECTORS MOVED HERE FROM CBIOS
;******************************************************

BIOS$JT	EQU  $+OFFSET!	JMP	0	;COLD BOOT.            
			JMP	0	;WARM BOOT.        
			JMP	0	;CONSOLE STATUS.   
			JMP	0	;CONSOLE INPUT.    
			JMP	0	;CONSOLE OUTPUT.   
B$LIST	EQU  $+OFFSET!	JMP	0	;LISTING OUTPUT.   
B$PNCH	EQU  $+OFFSET!	JMP	0	;PUNCH OUTPUT.     
B$PRDR	EQU  $+OFFSET!	JMP	0	;READER INPUT.     
B$HOME	EQU  $+OFFSET!	JMP	0	;RESTORE DRIVE.    
B$PICK	EQU  $+OFFSET!	JMP	0	;SELECT DRIVE.     
B$TRAK	EQU  $+OFFSET!	JMP	0	;SELECT TRACK.     
B$SECT	EQU  $+OFFSET!	JMP	0	;SELECT SECTOR.    
B$ADDR	EQU  $+OFFSET!	JMP	0	;SET ADDRESS.
B$READ	EQU  $+OFFSET!	JMP	0	;READ A SECTOR.    
B$WRIT	EQU  $+OFFSET!	JMP	0	;WRITE A SECTOR.   
B$LSTS	EQU  $+OFFSET!	JMP	0	;CHECK LIST STAT.
B$TRNS	EQU  $+OFFSET!	JMP	0	;TRANSLATE SECT. 

BIOS$JS	EQU  $+OFFSET - BIOS$JT

;******************************************************
; STORAGE
;******************************************************

T$DRIV	EQU	$+OFFSET!  DB	0	;SELECTED DRIVE.
T$TRAK	EQU	$+OFFSET!  DB	0	;SELECTED TRACK.
T$SECT	EQU	$+OFFSET!  DB	0	;SELECTED SECT.
T$ADDR	EQU	$+OFFSET!  DW	0	;TRANSFER ADDR.

RM$DPH	EQU	$+OFFSET

	DW	0,0		;DISK PARAMETER HDR
	DW	0,0
	DW	DIR$BF,RM$PBK
	DW	X$CSV,X$ALV

X$CSV	EQU	$+OFFSET!	DS	0	
X$ALV	EQU	$+OFFSET!	DS	33

;******************************************************
; SECTOR AND DIRECTORY BUFFER AREA
;******************************************************

SEC$BF	EQU	$+OFFSET!	DS	SEC$SZ
DIR$BF	EQU	$+OFFSET!	DS	SEC$SZ

;******************************************************

BLK$ND	EQU	$		;BLOCK END ADDRESS.
BLK$SZ	EQU	BLK$ND-BLK$BG+1	;BLOCK SIZE CALC.

;******************************************************
	END
